{.$DEFINE TESTSTATUS}
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DMailboxes, agsocket, msmb, Dimap4, Buttons, ComCtrls;

type
  TForm1 = class(TForm)
    IC: TImap4Client;
    CurrentMailboxes: TMailboxList;
    UntaggedMemo: TMemo;
    PageControl1: TPageControl;
    LoginSheet: TTabSheet;
    Label1: TLabel;
    ServerEdit: TEdit;
    UserEdit: TEdit;
    PassEdit: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    MailboxSheet: TTabSheet;
    MailboxEdit: TEdit;
    MailBoxesTree: TTreeView;
    MailboxImageList: TImageList;
    ListMailboxesButton: TSpeedButton;
    OpenMailboxButton: TSpeedButton;
    DefaultPrefix: TEdit;
    ConnectButton: TSpeedButton;
    DisconnectButton: TSpeedButton;
    CreateMailboxButton: TSpeedButton;
    DeleteMailboxButton: TSpeedButton;
    ExpungeButton: TSpeedButton;
    CancelButton: TSpeedButton;
    RetrieveButton: TSpeedButton;
    ListView1: TListView;
    ImageList1: TImageList;
    StatusBtn: TSpeedButton;
    TotalLab: TLabel;
    RecentLab: TLabel;
    UnSeenLab: TLabel;
    UIDNextLab: TLabel;
    UIDValidityLab: TLabel;
    procedure ICResponse(Sender: TObject; ResponseText: String);
    procedure ICAlert(Sender: TObject; ResponseText: String);
    procedure LogInBtnClick(Sender: TObject);
    procedure LogOutBtnClick(Sender: TObject);
    procedure OpenMailboxButtonClick(Sender: TObject);
    procedure ICListMailboxes(Sender: TObject; Item: String);
    procedure ListMailboxesButtonClick(Sender: TObject);
    procedure MailBoxesTreeChange(Sender: TObject; Node: TTreeNode);
    procedure MailBoxesTreeChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    procedure MailBoxesTreeExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure ICCommandInProgress(Sender: TObject);
    procedure ICCommandFinished(Sender: TObject);
    procedure ICServerSayBye(Sender: TObject);
    procedure CreateMailboxButtonClick(Sender: TObject);
    procedure DeleteMailboxButtonClick(Sender: TObject);
    procedure ExpungeButtonClick(Sender: TObject);
    procedure RetrieveButtonClick(Sender: TObject);
    procedure ICRetrieved(Sender: TObject; Num: Integer);
  private
    { Private declarations }
   procedure EnableControls;
   procedure DisableControls;
  public
    { Public declarations }
    CurrentRoot : TTreeNode;
    ParentName : String;
    IsClosing : Boolean;

    function GetFullMailboxName (Node : TTreeNode) : String;
  end;

var
  Form1: TForm1;

implementation

 uses StrFuncD; {Some useful string routines}

procedure TForm1.EnableControls;
begin
  ConnectButton.Enabled := not IC.OnLine;
  DisconnectButton.Enabled := IC.OnLine;

  ListMailboxesButton.Enabled:=true;
  RetrieveButton.Enabled:=true;
  CancelButton.Enabled:=false;

  CreateMailboxButton.Enabled := True;
  OpenMailboxButton.Enabled := True;
  DeleteMailboxButton.Enabled := True;
  ExpungeButton.Enabled := True;
end;

procedure TForm1.DisableControls;
begin
  ConnectButton.Enabled := False;
  DisconnectButton.Enabled := False;

  ListMailboxesButton.Enabled:=false;
  RetrieveButton.Enabled:=false;
  CancelButton.Enabled:=true;

  CreateMailboxButton.Enabled := False;
  OpenMailboxButton.Enabled := False;
  DeleteMailboxButton.Enabled := False;
  ExpungeButton.Enabled := False;
end;

function TForm1.GetFullMailboxName (Node : TTreeNode) : String;
 Var
  ParentNode : TTreeNode;
begin
 Result := Node.Text;
 ParentNode := Node.Parent;
 While ParentNode.Text<>ParentName do
  begin
   Result:=ParentNode.Text+TMailbox(Node.Data).Delimiter+Result;
   ParentNode:=ParentNode.Parent;
  end;
end;

{$R *.DFM}

procedure TForm1.ICResponse(Sender: TObject;
  ResponseText: String);
begin
 UntaggedMemo.Lines.Add (ResponseText);
end;

procedure TForm1.ICAlert(Sender: TObject; ResponseText: String);
begin
 ShowMessage (ResponseText);
end;

procedure TForm1.LogInBtnClick(Sender: TObject);
begin
 IC.Server := ServerEdit.Text;
 IC.UserName := UserEdit.Text;
 IC.Password := PassEdit.Text;
 IC.OpenConnection;
 IC.Capability;

 ParentName := UserEdit.Text + '@' + ServerEdit.Text;
 if DefaultPrefix.Text<>'' then
  ParentName := ParentName + ' ' + DefaultPrefix.Text;

 EnableControls;
 MailboxSheet.TabVisible := True;
 PageControl1.ActivePage := MailboxSheet;
end;

procedure TForm1.LogOutBtnClick(Sender: TObject);
begin
 IsClosing := True;
 IC.CloseConnection;
 IsClosing := False;
 DisableControls;
 ConnectButton.Enabled := True;
 MailboxSheet.TabVisible := False;
end;

procedure TForm1.OpenMailboxButtonClick(Sender: TObject);
begin
 if IC.OnLine then
  IC.MailBox := MailboxEdit.Text;
end;

procedure TForm1.ICListMailboxes(Sender: TObject; Item: String);
Var
 ShortName : String;
 CurNode : TTreeNode;
begin
   ShortName := GetRight (Item,CurrentMailboxes[Item].Delimiter);
   CurNode := MailBoxesTree.Items.AddChildObject(CurrentRoot, ShortName, CurrentMailboxes[Item]);

   if NoInferiors in CurrentMailboxes[Item].Options then
    CurrentMailboxes[Item].Tag := 1
   else
    begin
     MailBoxesTree.Items.AddChildObject(CurNode, '', NIL);
     CurrentMailboxes[Item].Tag := 0; {Item was not expanded}
    end;

   With CurNode do
   begin
    if CurrentMailboxes[Text].Subscribed then
     ImageIndex := 2
    else
     if Noselect in CurrentMailboxes[Text].Options then {Pos('\NoSelect',AllMailboxes[Text].Flags)<>0}
      ImageIndex := 3 {StateIndex}
     else
      ImageIndex := 1; {StateIndex}

    SelectedIndex := 0;
   end;
end;

procedure TForm1.ListMailboxesButtonClick(Sender: TObject);
begin
 try
  {MailBoxesTree.Items.BeginUpdate;}
  MailBoxesTree.Items.Clear;
  With MailBoxesTree.Items.Add(MailBoxesTree.Items.GetFirstNode,ParentName) do
   begin
    ImageIndex := 0; {StateIndex}
    SelectedIndex := 0;
   end;

   {!!!}
  CurrentRoot := MailBoxesTree.Items.GetFirstNode; {Add all mailboxes to root node}

  With IC do
   begin
    ListSubscribedMailBoxes (DefaultPrefix.Text, '%');
    ListMailBoxes (DefaultPrefix.Text, '%');
   end;

 finally
  {MailBoxesTree.Items.EndUpdate;}
 end;
end;

procedure TForm1.MailBoxesTreeChange(Sender: TObject; Node: TTreeNode);
 Var
  FullName : String;

  Mbx : TMailbox;
begin
 if Node.Text<>ParentName then
  begin
   FullName := GetFullMailboxName(Node);
   try
    MailBoxEdit.Text := FullName;
    OpenMailboxButtonClick(Self);

    Mbx := TMailbox(Node.Data);
    TotalLab.Caption:=TotalLab.Hint+' '+IntToStr(Mbx.Total);
    RecentLab.Caption:=RecentLab.Hint+' '+IntToStr(Mbx.Recent);
    UnSeenLab.Caption:=UnSeenLab.Hint+' '+IntToStr(Mbx.UnSeen);
    UIDNextLab.Caption:=UIDNextLab.Hint+' '+IntToStr(Mbx.UIDNext);
    UIDValidityLab.Caption:=UIDValidityLab.Hint+' '+IntToStr(Mbx.UIDValidity);

   except
    {StatusBar1.Panels[1].Text:='Can not open '+FullName;}
   end; {try}
  end; {if}
end;

procedure TForm1.MailBoxesTreeChanging(Sender: TObject; Node: TTreeNode;
  var AllowChange: Boolean);
begin
 AllowChange := (Node.Text<>ParentName);
end;

procedure TForm1.MailBoxesTreeExpanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
Var
 Mbx : TMailbox;
begin
 if Node.Text = ParentName then
  Exit; {Always allow expansion}

 Mbx := TMailbox(Node.Data);
 if Mbx=NIL then
  raise Exception.Create ('Internal error');

 if Mbx.Tag <> 0 then {Already expanded}
  Exit;

 Node.DeleteChildren;

 try
  {MailBoxesTree.Items.BeginUpdate;}
   {!!!}
  CurrentRoot := Node;

  With IC do
   begin
{$IFDEF TESTSTATUS}
    QueryMailBox (GetFullMailboxName(Node),[isaMESSAGES,
                                            isaRECENT,
                                            isaUIDNEXT,
                                            isaUIDVALIDITY,
                                            isaUNSEEN]);
{$ENDIF}

    ListSubscribedMailBoxes (DefaultPrefix.Text, GetFullMailboxName(Node)+Mbx.Delimiter+'%');
    ListMailBoxes (DefaultPrefix.Text, GetFullMailboxName(Node)+Mbx.Delimiter+'%');
   end;

  Mbx.Tag := 1; {Expanded}

 finally
  {MailBoxesTree.Items.EndUpdate;}
 end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 if IC.Online then
  try
   IC.CloseConnection;
  except
  end;
end;

procedure TForm1.CancelButtonClick(Sender: TObject);
begin
 IC.Cancel; {Cancel calls Close}

 DisableControls;
 MailBoxesTree.Items.Clear;
 CurrentMailboxes.Clear;
 ConnectButton.Enabled := True;
end;

procedure TForm1.ICCommandInProgress(Sender: TObject);
begin
 DisableControls;
end;

procedure TForm1.ICCommandFinished(Sender: TObject);
begin
 EnableControls;
end;

procedure TForm1.ICServerSayBye(Sender: TObject);
begin
 IC.CloseConnection;
 DisableControls;
 {MailBoxesTree.Items.Clear;
 CurrentMailboxes.Clear;}
 ConnectButton.Enabled := True;

 if not IsClosing then
  if MessageDlg('Server closed socked connection. Reconnect?',
                 mtConfirmation, [mbYes, mbNo], 0) = id_Yes then
   begin
    IC.OpenConnection;
    IC.Capability;
   end;
end;

procedure TForm1.CreateMailboxButtonClick(Sender: TObject);
begin
 With IC do
  begin
   CreateMailbox (MailBoxEdit.Text);
   {StatusBar1.Panels[1].Text:='Mailbox Created';}
  end;

 // Update Tree view
 ListMailboxesButtonClick(Sender);
end;

procedure TForm1.DeleteMailboxButtonClick(Sender: TObject);
begin
 With IC do
  begin
   DeleteMailbox (MailBoxEdit.Text);
   {StatusBar1.Panels[1].Text:='Mailbox Deleted';}
  end;

 // Update Tree View:
 ListMailboxesButtonClick(Sender);
end;

procedure TForm1.ExpungeButtonClick(Sender: TObject);
begin
  IC.ExpungeDeleted;
end;

procedure TForm1.RetrieveButtonClick(Sender: TObject);
begin
  IC.MessageList.Clear;
  ListView1.Items.Clear;
  ListView1.Cursor:=crHourGlass;
  DisableControls;
  try
    IC.RetrieveMessageSet ('1:*','(FLAGS INTERNALDATE RFC822.SIZE ENVELOPE)');
  finally
    EnableControls;
    ListView1.Cursor:=crDefault;
  end;
end;

procedure TForm1.ICRetrieved(Sender: TObject; Num: Integer);
var
  i : Integer;
  Item : TListItem;
begin
  i:=IC.MessageList.Count-1;
  Item:=ListView1.Items.Add;
  Item.Caption:=IC.MessageList[i].Subject;
  if Item.Caption='' then Item.Caption:='<No Subject>';
  Item.ImageIndex:=0;
end;


end.
